@@ -1,15 +1,9 @@
 """Test `uniprops`."""
-from __future__ import unicode_literals
 import unittest
 import sys
 from backrefs import uniprops
 
-PY3 = (3, 0) <= sys.version_info < (4, 0)
-
-if PY3:
-    binary_type = bytes  # noqa
-else:
-    binary_type = str  # noqa
+PY37 = (3, 7) <= sys.version_info
 
 
 class TestUniprops(unittest.TestCase):
@@ -51,6 +45,18 @@ class TestUniprops(unittest.TestCase):
         result = uniprops.get_unicode_property('^latin', 'sc')
         self.assertEqual(result, uniprops.unidata.unicode_scripts['^latin'])
 
+    def test_script_extensions(self):
+        """Test `script extensions` Category."""
+
+        result = uniprops.get_unicode_property('kana', 'scx')
+        self.assertEqual(result, uniprops.unidata.unicode_script_extensions['katakana'])
+
+    def test_inverse_script_extensions(self):
+        """Test inverse `script extensions` Category."""
+
+        result = uniprops.get_unicode_property('^kana', 'scx')
+        self.assertEqual(result, uniprops.unidata.unicode_script_extensions['^katakana'])
+
     def test_bidi(self):
         """Test `bidi class` Category."""
 
@@ -63,6 +69,18 @@ class TestUniprops(unittest.TestCase):
         result = uniprops.get_unicode_property('^en', 'bc')
         self.assertEqual(result, uniprops.unidata.unicode_bidi_classes['^en'])
 
+    def test_bidi_paired_bracket_type(self):
+        """Test `bidi paired bracket type` Category."""
+
+        result = uniprops.get_unicode_property('o', 'bpt')
+        self.assertEqual(result, uniprops.unidata.unicode_bidi_paired_bracket_type['o'])
+
+    def test_inverse_bidi_paired_bracket_type(self):
+        """Test inverse `bidi paired bracket type` Category."""
+
+        result = uniprops.get_unicode_property('^o', 'bpt')
+        self.assertEqual(result, uniprops.unidata.unicode_bidi_paired_bracket_type['^o'])
+
     def test_decompostion(self):
         """Test `decomposition type` Category."""
 
@@ -99,6 +117,30 @@ class TestUniprops(unittest.TestCase):
         result = uniprops.get_unicode_property('^f', 'ea')
         self.assertEqual(result, uniprops.unidata.unicode_east_asian_width['^f'])
 
+    def test_indicpositionalcategory(self):
+        """Test `indic positional/matra category` Category."""
+
+        result = uniprops.get_unicode_property('top', 'inpc')
+        self.assertEqual(result, uniprops.unidata.unicode_indic_positional_category['top'])
+
+    def test_inverse_indicpositionalcategory(self):
+        """Test inverse `indic positional/matra category` Category."""
+
+        result = uniprops.get_unicode_property('^top', 'inpc')
+        self.assertEqual(result, uniprops.unidata.unicode_indic_positional_category['^top'])
+
+    def test_indicsylabiccategory(self):
+        """Test `indic syllabic category` Category."""
+
+        result = uniprops.get_unicode_property('bindu', 'insc')
+        self.assertEqual(result, uniprops.unidata.unicode_indic_syllabic_category['bindu'])
+
+    def test_inverse_indicsyllabiccategory(self):
+        """Test inverse `indic syllabic category` Category."""
+
+        result = uniprops.get_unicode_property('^bindu', 'insc')
+        self.assertEqual(result, uniprops.unidata.unicode_indic_syllabic_category['^bindu'])
+
     def test_hangulsyllabletype(self):
         """Test `hangul syllable type` Category."""
 
@@ -267,6 +309,30 @@ class TestUniprops(unittest.TestCase):
         result = uniprops.get_unicode_property('^y', 'nfkdqc')
         self.assertEqual(result, uniprops.unidata.unicode_nfkd_quick_check['^y'])
 
+    def test_vertical_orientation(self):
+        """Test `vertical orientation` Category."""
+
+        if PY37:
+            result = uniprops.get_unicode_property('u', 'vo')
+            self.assertEqual(result, uniprops.unidata.unicode_vertical_orientation['u'])
+        else:
+            with self.assertRaises(ValueError) as e:
+                uniprops.get_unicode_property('u', 'vo')
+
+            self.assertTrue(str(e), 'Invalid Unicode property!')
+
+    def test_inverse_vertical_orientation(self):
+        """Test inverse `vertical orientation` Category."""
+
+        if PY37:
+            result = uniprops.get_unicode_property('^u', 'vo')
+            self.assertEqual(result, uniprops.unidata.unicode_vertical_orientation['^u'])
+        else:
+            with self.assertRaises(ValueError) as e:
+                uniprops.get_unicode_property('^u', 'vo')
+
+            self.assertTrue(str(e), 'Invalid Unicode property!')
+
     def test_gc_simple(self):
         """Test `gc` simple Category."""
 
@@ -283,25 +349,37 @@ class TestUniprops(unittest.TestCase):
         """Test `isscript` Category."""
 
         result = uniprops.get_unicode_property('islatin')
-        self.assertEqual(result, uniprops.unidata.unicode_scripts['latin'])
+        self.assertEqual(result, uniprops.unidata.unicode_script_extensions['latin'])
 
     def test_inverse_is_script(self):
         """Test inverse `isscript` Category."""
 
         result = uniprops.get_unicode_property('^islatin')
-        self.assertEqual(result, uniprops.unidata.unicode_scripts['^latin'])
+        self.assertEqual(result, uniprops.unidata.unicode_script_extensions['^latin'])
+
+    def test_is_binary(self):
+        """Test `isbinary` Category."""
+
+        result = uniprops.get_unicode_property('isalphabetic')
+        self.assertEqual(result, uniprops.unidata.unicode_binary['alphabetic'])
+
+    def test_inverse_is_binary(self):
+        """Test inverse `isbinary` Category."""
+
+        result = uniprops.get_unicode_property('^isalphabetic')
+        self.assertEqual(result, uniprops.unidata.unicode_binary['^alphabetic'])
 
     def test_script_simple(self):
         """Test script simple Category."""
 
         result = uniprops.get_unicode_property('latin')
-        self.assertEqual(result, uniprops.unidata.unicode_scripts['latin'])
+        self.assertEqual(result, uniprops.unidata.unicode_script_extensions['latin'])
 
     def test_inverse_script_simple(self):
         """Test inverse script simple Category."""
 
         result = uniprops.get_unicode_property('^latin')
-        self.assertEqual(result, uniprops.unidata.unicode_scripts['^latin'])
+        self.assertEqual(result, uniprops.unidata.unicode_script_extensions['^latin'])
 
     def test_in_block(self):
         """Test `inblock` Category."""
@@ -394,13 +472,13 @@ class TestUniprops(unittest.TestCase):
     def test_posix_binary(self):
         """Test binary `posix` Category."""
 
-        result = uniprops.get_posix_property('punct', uniprops.POSIX_BINARY)
+        result = uniprops.get_posix_property('punct', uniprops.POSIX_BYTES)
         self.assertEqual(result, uniprops.unidata.ascii_posix_properties['punct'])
 
     def test_inverse_posix_binary(self):
         """Test inverse binary `posix` Category."""
 
-        result = uniprops.get_posix_property('^punct', uniprops.POSIX_BINARY)
+        result = uniprops.get_posix_property('^punct', uniprops.POSIX_BYTES)
         self.assertEqual(result, uniprops.unidata.ascii_posix_properties['^punct'])
 
     def test_posix(self):
