@@ -1,61 +1,40 @@
-"""
-Generate a Unicode prop table for Python narrow and wide builds.
-
-Narrow builds will stop at `0xffff`.
-"""
+"""Generate a Unicode prop table for Python builds."""
 from __future__ import unicode_literals
 import sys
-import struct
 import unicodedata
 import codecs
 import os
 import re
 
-__version__ = '3.0.0'
+__version__ = '4.2.0'
 
-UNIVERSION = unicodedata.unidata_version
-UNIVERSION_INFO = tuple([int(x) for x in UNIVERSION.split('.')])
+UNIVERSION = None
+UNIVERSION_INFO = None
 HOME = os.path.dirname(os.path.abspath(__file__))
 MAXUNICODE = sys.maxunicode
 MAXASCII = 0xFF
-NARROW = sys.maxunicode == 0xFFFF
+GROUP_ESCAPES = frozenset([ord(x) for x in '-&[\\]^|~'])
 
-# Compatibility
-PY3 = sys.version_info >= (3, 0) and sys.version_info[0:2] < (4, 0)
-if NARROW:
-    UNICODE_RANGE = (0x0000, 0xFFFF)
-else:
-    UNICODE_RANGE = (0x0000, 0x10FFFF)
+UNICODE_RANGE = (0x0000, 0x10FFFF)
 ASCII_RANGE = (0x00, 0xFF)
-if PY3:
-    unichar = chr  # noqa
-else:
-    unichar = unichr  # noqa
 
 ALL_CHARS = set([x for x in range(UNICODE_RANGE[0], UNICODE_RANGE[1] + 1)])
 ALL_ASCII = set([x for x in range(ASCII_RANGE[0], ASCII_RANGE[1] + 1)])
 HEADER = '''\
-"""Unicode Properties from Unicode version %s (autogen)."""
+"""Unicode Properties from Unicode version {} (autogen)."""
 from __future__ import unicode_literals
 
-''' % UNIVERSION
-
-
-def uchr(i):
-    """Allow getting Unicode character on narrow Python builds."""
-
-    try:
-        return unichar(i)
-    except ValueError:
-        return struct.pack('i', i).decode('utf-32')
+'''
 
 
 def uniformat(value):
     """Convert a Unicode char."""
 
-    # Escape #^-\]
-    if value in (0x55, 0x5c, 0x5d):
-        c = "\\u%04x\\u%04x" % (0x5c, value)
+    if value in GROUP_ESCAPES:
+        # Escape characters that are (or will be in the future) problematic
+        c = "\\x%02x\\x%02x" % (0x5c, value)
+    elif value <= 0xFF:
+        c = "\\x%02x" % value
     elif value <= 0xFFFF:
         c = "\\u%04x" % value
     else:
@@ -68,39 +47,34 @@ def format_name(text):
     return text.strip().lower().replace(' ', '').replace('-', '').replace('_', '')
 
 
-def binaryformat(value):
-    """Convert a binary value."""
+def bytesformat(value):
+    """Convert a bytes value."""
 
-    # Escape #^-\]
-    if value in (0x55, 0x5c, 0x5d):
+    if value in GROUP_ESCAPES:
+        # Escape characters that are (or will be in the future) problematic
         c = "\\x%02x\\x%02x" % (0x5c, value)
     else:
         c = "\\x%02x" % value
     return c
 
 
-def create_span(unirange, binary=False):
+def create_span(unirange, is_bytes=False):
     """Clamp the Unicode range."""
 
     if len(unirange) < 2:
         unirange.append(unirange[0])
-    if binary:
+    if is_bytes:
         if unirange[0] > MAXASCII:
             return None
         if unirange[1] > MAXASCII:
             unirange[1] = MAXASCII
-    elif NARROW:
-        if unirange[0] > MAXUNICODE:
-            return None
-        if unirange[1] > MAXUNICODE:
-            unirange[1] = MAXUNICODE
     return [x for x in range(unirange[0], unirange[1] + 1)]
 
 
-def not_explicitly_defined(table, name, binary=False):
+def not_explicitly_defined(table, name, is_bytes=False):
     """Compose a table with the specified entry name of values not explicitly defined."""
 
-    all_chars = ALL_ASCII if binary else ALL_CHARS
+    all_chars = ALL_ASCII if is_bytes else ALL_CHARS
     s = set()
     for k, v in table.items():
         s.update(v)
@@ -110,16 +84,16 @@ def not_explicitly_defined(table, name, binary=False):
         table[name] = list(all_chars - s)
 
 
-def char2range(d, binary=False, invert=True):
+def char2range(d, is_bytes=False, invert=True):
     """Convert the characters in the dict to a range in string form."""
 
-    fmt = binaryformat if binary else uniformat
-    maxrange = MAXASCII if binary else MAXUNICODE
+    fmt = bytesformat if is_bytes else uniformat
+    maxrange = MAXASCII if is_bytes else MAXUNICODE
 
     for k1 in sorted(d.keys()):
         v1 = d[k1]
         if not isinstance(v1, list):
-            char2range(v1, binary=binary, invert=invert)
+            char2range(v1, is_bytes=is_bytes, invert=invert)
         else:
             inverted = k1.startswith('^')
             v1.sort()
@@ -182,15 +156,15 @@ def gen_blocks(output, ascii_props=False, append=False, prefix=""):
 
     with codecs.open(output, 'a' if append else 'w', 'utf-8') as f:
         if not append:
-            f.write(HEADER)
+            f.write(HEADER.format(UNIVERSION))
         f.write('%s_blocks = {' % prefix)
         no_block = []
         last = -1
 
         max_range = MAXASCII if ascii_props else MAXUNICODE
-        formatter = binaryformat if ascii_props else uniformat
+        formatter = bytesformat if ascii_props else uniformat
 
-        with open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'Blocks.txt'), 'r') as uf:
+        with codecs.open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'Blocks.txt'), 'r', 'utf-8') as uf:
             for line in uf:
                 if not line.startswith('#'):
                     data = line.split(';')
@@ -239,16 +213,16 @@ def gen_blocks(output, ascii_props=False, append=False, prefix=""):
 
 
 def gen_ccc(output, ascii_props=False, append=False, prefix=""):
-    """Generate canonical combining class."""
+    """Generate `canonical combining class` property."""
 
     obj = {}
-    with open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'DerivedCombiningClass.txt'), 'r') as uf:
+    with codecs.open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'DerivedCombiningClass.txt'), 'r', 'utf-8') as uf:
         for line in uf:
             if not line.startswith('#'):
                 data = line.split('#')[0].split(';')
                 if len(data) < 2:
                     continue
-                span = create_span([int(i, 16) for i in data[0].strip().split('..')], binary=ascii_props)
+                span = create_span([int(i, 16) for i in data[0].strip().split('..')], is_bytes=ascii_props)
                 if span is None:
                     continue
                 name = format_name(data[1])
@@ -266,15 +240,15 @@ def gen_ccc(output, ascii_props=False, append=False, prefix=""):
         s = set(obj[name])
         obj[name] = sorted(s)
 
-    not_explicitly_defined(obj, '0', binary=ascii_props)
+    not_explicitly_defined(obj, '0', is_bytes=ascii_props)
 
     # Convert characters values to ranges
-    char2range(obj, binary=ascii_props)
+    char2range(obj, is_bytes=ascii_props)
 
     with codecs.open(output, 'a' if append else 'w', 'utf-8') as f:
         if not append:
-            f.write(HEADER)
-        # Write out the unicode properties
+            f.write(HEADER.format(UNIVERSION))
+        # Write out the Unicode properties
         f.write('%s_canonical_combining_class = {\n' % prefix)
         count = len(obj) - 1
         i = 0
@@ -287,17 +261,114 @@ def gen_ccc(output, ascii_props=False, append=False, prefix=""):
             i += 1
 
 
-def gen_enum(file_name, obj_name, output, field=1, notexplicit=None, ascii_props=False, append=False, prefix=""):
-    """Generate generic enum."""
+def gen_scripts(
+    file_name, file_name_ext, obj_name, obj_ext_name, output, output_ext,
+    field=1, notexplicit=None, ascii_props=False, append=False, prefix=""
+):
+    """Generate `script` property."""
 
     obj = {}
-    with open(os.path.join(HOME, 'unicodedata', UNIVERSION, file_name), 'r') as uf:
+    obj2 = {}
+    aliases = {}
+    with codecs.open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'PropertyValueAliases.txt'), 'r', 'utf-8') as uf:
+        for line in uf:
+            if line.startswith('sc ;'):
+                values = line.split(';')
+                aliases[format_name(values[1].strip())] = format_name(values[2].strip())
+
+    with codecs.open(os.path.join(HOME, 'unicodedata', UNIVERSION, file_name_ext), 'r', 'utf-8') as uf:
         for line in uf:
             if not line.startswith('#'):
                 data = line.split('#')[0].split(';')
                 if len(data) < 2:
                     continue
-                span = create_span([int(i, 16) for i in data[0].strip().split('..')], binary=ascii_props)
+                exts = [aliases[format_name(n)] for n in data[1].strip().split(' ')]
+                span = create_span([int(i, 16) for i in data[0].strip().split('..')], is_bytes=ascii_props)
+                for ext in exts:
+                    if ext not in obj2:
+                        obj2[ext] = []
+                    if span is None:
+                        continue
+
+                    obj2[ext].extend(span)
+
+    with codecs.open(os.path.join(HOME, 'unicodedata', UNIVERSION, file_name), 'r', 'utf-8') as uf:
+        for line in uf:
+            if not line.startswith('#'):
+                data = line.split('#')[0].split(';')
+                if len(data) < 2:
+                    continue
+                span = create_span([int(i, 16) for i in data[0].strip().split('..')], is_bytes=ascii_props)
+                name = format_name(data[1])
+                if name not in obj:
+                    obj[name] = []
+                if name not in obj2:
+                    obj2[name] = []
+
+                if span is None:
+                    continue
+
+                obj[name].extend(span)
+                obj2[name].extend(span)
+
+    for name in list(obj.keys()):
+        s = set(obj[name])
+        obj[name] = sorted(s)
+
+    for name in list(obj2.keys()):
+        s = set(obj2[name])
+        obj2[name] = sorted(s)
+
+    if notexplicit:
+        not_explicitly_defined(obj, notexplicit, is_bytes=ascii_props)
+        not_explicitly_defined(obj2, notexplicit, is_bytes=ascii_props)
+
+    # Convert characters values to ranges
+    char2range(obj, is_bytes=ascii_props)
+    char2range(obj2, is_bytes=ascii_props)
+
+    with codecs.open(output, 'a' if append else 'w', 'utf-8') as f:
+        if not append:
+            f.write(HEADER.format(UNIVERSION))
+        # Write out the Unicode properties
+        f.write('%s_%s = {\n' % (prefix, obj_name))
+        count = len(obj) - 1
+        i = 0
+        for k1, v1 in sorted(obj.items()):
+            f.write('    "%s": "%s"' % (k1, v1))
+            if i == count:
+                f.write('\n}\n')
+            else:
+                f.write(',\n')
+            i += 1
+
+    with codecs.open(output_ext, 'a' if append else 'w', 'utf-8') as f:
+        if not append:
+            f.write(HEADER.format(UNIVERSION))
+        # Write out the Unicode properties
+        f.write('%s_%s = {\n' % (prefix, obj_ext_name))
+        count = len(obj2) - 1
+        i = 0
+        for k1, v1 in sorted(obj2.items()):
+            f.write('    "%s": "%s"' % (k1, v1))
+            if i == count:
+                f.write('\n}\n')
+            else:
+                f.write(',\n')
+            i += 1
+
+
+def gen_enum(file_name, obj_name, output, field=1, notexplicit=None, ascii_props=False, append=False, prefix=""):
+    """Generate generic enum."""
+
+    obj = {}
+    with codecs.open(os.path.join(HOME, 'unicodedata', UNIVERSION, file_name), 'r', 'utf-8') as uf:
+        for line in uf:
+            if not line.startswith('#'):
+                data = line.split('#')[0].split(';')
+                if len(data) < 2:
+                    continue
+                span = create_span([int(i, 16) for i in data[0].strip().split('..')], is_bytes=ascii_props)
                 name = format_name(data[field])
                 if name not in obj:
                     obj[name] = []
@@ -312,15 +383,15 @@ def gen_enum(file_name, obj_name, output, field=1, notexplicit=None, ascii_props
         obj[name] = sorted(s)
 
     if notexplicit:
-        not_explicitly_defined(obj, notexplicit, binary=ascii_props)
+        not_explicitly_defined(obj, notexplicit, is_bytes=ascii_props)
 
     # Convert characters values to ranges
-    char2range(obj, binary=ascii_props)
+    char2range(obj, is_bytes=ascii_props)
 
     with codecs.open(output, 'a' if append else 'w', 'utf-8') as f:
         if not append:
-            f.write(HEADER)
-        # Write out the unicode properties
+            f.write(HEADER.format(UNIVERSION))
+        # Write out the Unicode properties
         f.write('%s_%s = {\n' % (prefix, obj_name))
         count = len(obj) - 1
         i = 0
@@ -334,17 +405,17 @@ def gen_enum(file_name, obj_name, output, field=1, notexplicit=None, ascii_props
 
 
 def gen_age(output, ascii_props=False, append=False, prefix=""):
-    """Generate Age."""
+    """Generate `age` property."""
 
     obj = {}
     all_chars = ALL_ASCII if ascii_props else ALL_CHARS
-    with open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'DerivedAge.txt'), 'r') as uf:
+    with codecs.open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'DerivedAge.txt'), 'r', 'utf-8') as uf:
         for line in uf:
             if not line.startswith('#'):
                 data = line.split('#')[0].split(';')
                 if len(data) < 2:
                     continue
-                span = create_span([int(i, 16) for i in data[0].strip().split('..')], binary=ascii_props)
+                span = create_span([int(i, 16) for i in data[0].strip().split('..')], is_bytes=ascii_props)
                 name = format_name(data[1])
 
                 if name not in obj:
@@ -365,12 +436,12 @@ def gen_age(output, ascii_props=False, append=False, prefix=""):
         obj[name] = sorted(s)
 
     # Convert characters values to ranges
-    char2range(obj, binary=ascii_props)
+    char2range(obj, is_bytes=ascii_props)
 
     with codecs.open(output, 'a' if append else 'w', 'utf-8') as f:
         if not append:
             f.write(HEADER)
-        # Write out the unicode properties
+        # Write out the Unicode properties
         f.write('%s_age = {\n' % prefix)
         count = len(obj) - 1
         i = 0
@@ -384,12 +455,13 @@ def gen_age(output, ascii_props=False, append=False, prefix=""):
 
 
 def gen_nf_quick_check(output, ascii_props=False, append=False, prefix=""):
-    """Generate binary properties."""
+    """Generate quick check properties."""
 
     categories = []
     nf = {}
     all_chars = ALL_ASCII if ascii_props else ALL_CHARS
-    with open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'DerivedNormalizationProps.txt'), 'r') as uf:
+    file_name = os.path.join(HOME, 'unicodedata', UNIVERSION, 'DerivedNormalizationProps.txt')
+    with codecs.open(file_name, 'r', 'utf-8') as uf:
         for line in uf:
             if not line.startswith('#'):
                 data = line.split('#')[0].split(';')
@@ -397,7 +469,7 @@ def gen_nf_quick_check(output, ascii_props=False, append=False, prefix=""):
                     continue
                 if not data[1].strip().lower().endswith('_qc'):
                     continue
-                span = create_span([int(i, 16) for i in data[0].strip().split('..')], binary=ascii_props)
+                span = create_span([int(i, 16) for i in data[0].strip().split('..')], is_bytes=ascii_props)
                 if span is None:
                     continue
                 name = format_name(data[1][:-3] + 'quickcheck')
@@ -422,13 +494,13 @@ def gen_nf_quick_check(output, ascii_props=False, append=False, prefix=""):
             nf[k1][name] = sorted(s)
 
     # Convert characters values to ranges
-    char2range(nf, binary=ascii_props)
+    char2range(nf, is_bytes=ascii_props)
 
     with codecs.open(output, 'a' if append else 'w', 'utf-8') as f:
         if not append:
-            f.write(HEADER)
+            f.write(HEADER.format(UNIVERSION))
         for key, value in sorted(nf.items()):
-            # Write out the unicode properties
+            # Write out the Unicode properties
             f.write('%s_%s = {\n' % (prefix, key.replace('quickcheck', '_quick_check')))
             count = len(value) - 1
             i = 0
@@ -447,14 +519,18 @@ def gen_binary(table, output, ascii_props=False, append=False, prefix=""):
     """Generate binary properties."""
 
     categories = []
-    binary_props = (
+    binary_props = [
         ('DerivedCoreProperties.txt', None),
         ('PropList.txt', None),
         ('DerivedNormalizationProps.txt', ('Changes_When_NFKC_Casefolded', 'Full_Composition_Exclusion'))
-    )
+    ]
+
+    if UNIVERSION_INFO >= (13, 0, 0):
+        binary_props.append(('emoji-data.txt', None))
+
     binary = {}
     for filename, include in binary_props:
-        with open(os.path.join(HOME, 'unicodedata', UNIVERSION, filename), 'r') as uf:
+        with codecs.open(os.path.join(HOME, 'unicodedata', UNIVERSION, filename), 'r', 'utf-8') as uf:
             for line in uf:
                 if not line.startswith('#'):
                     data = line.split('#')[0].split(';')
@@ -462,7 +538,7 @@ def gen_binary(table, output, ascii_props=False, append=False, prefix=""):
                         continue
                     if include and data[1].strip() not in include:
                         continue
-                    span = create_span([int(i, 16) for i in data[0].strip().split('..')], binary=ascii_props)
+                    span = create_span([int(i, 16) for i in data[0].strip().split('..')], is_bytes=ascii_props)
                     name = format_name(data[1])
 
                     if name not in binary:
@@ -472,14 +548,14 @@ def gen_binary(table, output, ascii_props=False, append=False, prefix=""):
                         continue
                     binary[name].extend(span)
 
-    with open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'CompositionExclusions.txt'), 'r') as uf:
+    with codecs.open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'CompositionExclusions.txt'), 'r', 'utf-8') as uf:
         name = 'compositionexclusion'
         for line in uf:
             if not line.startswith('#'):
                 data = [x.strip() for x in line.split('#')[0] if x.strip()]
                 if not data:
                     continue
-                span = create_span([int(data[0], 16)], binary=ascii_props)
+                span = create_span([int(data[0], 16)], is_bytes=ascii_props)
                 if span is None:
                     continue
 
@@ -489,14 +565,14 @@ def gen_binary(table, output, ascii_props=False, append=False, prefix=""):
                 binary[name].extend(span)
                 binary['full' + name].extend(span)
 
-    with open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'UnicodeData.txt'), 'r') as uf:
+    with codecs.open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'UnicodeData.txt'), 'r', 'utf-8') as uf:
         name = 'bidimirrored'
         for line in uf:
             data = line.strip().split(';')
             if data:
                 if data[9].strip().lower() != 'y':
                     continue
-                span = create_span([int(data[0].strip(), 16)], binary=ascii_props)
+                span = create_span([int(data[0].strip(), 16)], is_bytes=ascii_props)
                 if span is None:
                     continue
 
@@ -512,12 +588,12 @@ def gen_binary(table, output, ascii_props=False, append=False, prefix=""):
     gen_uposix(table, binary)
 
     # Convert characters values to ranges
-    char2range(binary, binary=ascii_props)
+    char2range(binary, is_bytes=ascii_props)
 
     with codecs.open(output, 'a' if append else 'w', 'utf-8') as f:
         if not append:
-            f.write(HEADER)
-        # Write out the unicode properties
+            f.write(HEADER.format(UNIVERSION))
+        # Write out the Unicode properties
         f.write('%s_binary = {\n' % prefix)
         count = len(binary) - 1
         i = 0
@@ -533,11 +609,11 @@ def gen_binary(table, output, ascii_props=False, append=False, prefix=""):
 
 
 def gen_bidi(output, ascii_props=False, append=False, prefix=""):
-    """Generate bidi class properties."""
+    """Generate `bidi class` property."""
 
     bidi_class = {}
     max_range = MAXASCII if ascii_props else MAXUNICODE
-    with open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'UnicodeData.txt'), 'r') as uf:
+    with codecs.open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'UnicodeData.txt'), 'r', 'utf-8') as uf:
         for line in uf:
             data = line.strip().split(';')
             if data:
@@ -559,11 +635,11 @@ def gen_bidi(output, ascii_props=False, append=False, prefix=""):
         bidi_class[name] = sorted(s)
 
     # Convert characters values to ranges
-    char2range(bidi_class, binary=ascii_props)
+    char2range(bidi_class, is_bytes=ascii_props)
 
     with codecs.open(output, 'a' if append else 'w', 'utf-8') as f:
         if not append:
-            f.write(HEADER)
+            f.write(HEADER.format(UNIVERSION))
         f.write('%s_bidi_classes = {\n' % prefix)
         count = len(bidi_class) - 1
         i = 0
@@ -576,78 +652,78 @@ def gen_bidi(output, ascii_props=False, append=False, prefix=""):
             i += 1
 
 
-def gen_posix(output, binary=False, append=False, prefix=""):
-    """Generate the binary posix table and write out to file."""
+def gen_posix(output, is_bytes=False, append=False, prefix=""):
+    """Generate the bytes posix table and write out to file."""
 
     posix_table = {}
 
-    # Alnum: [a-zA-Z0-9]
+    # `Alnum: [a-zA-Z0-9]`
     s = set([x for x in range(0x30, 0x39 + 1)])
     s |= set([x for x in range(0x41, 0x5a + 1)])
     s |= set([x for x in range(0x61, 0x7a + 1)])
     posix_table["alnum"] = list(s)
 
-    # Alpha: [a-zA-Z]
+    # `Alpha: [a-zA-Z]`
     s = set([x for x in range(0x41, 0x5a)])
     s |= set([x for x in range(0x61, 0x7a)])
     posix_table["alpha"] = list(s)
 
-    # ASCII: [\x00-\x7F]
+    # `ASCII: [\x00-\x7F]`
     s = set([x for x in range(0, 0x7F + 1)])
     posix_table["ascii"] = list(s)
 
-    # Blank: [ \t]
+    # `Blank: [ \t]`
     s = set([0x20, 0x09])
     posix_table["blank"] = list(s)
 
-    # Cntrl: [\x00-\x1F\x7F]
+    # `Cntrl: [\x00-\x1F\x7F]`
     s = set([x for x in range(0, 0x1F + 1)] + [0x7F])
     posix_table["cntrl"] = list(s)
 
-    # Digit: [0-9]
+    # `Digit: [0-9]`
     s = set([x for x in range(0x30, 0x39 + 1)])
     posix_table["digit"] = list(s)
 
-    # Graph: [\x21-\x7E]
+    # `Graph: [\x21-\x7E]`
     s = set([x for x in range(0x21, 0x7E + 1)])
     posix_table["graph"] = list(s)
 
-    # Lower: [a-z]
+    # `Lower: [a-z]`
     s = set([x for x in range(0x61, 0x7a + 1)])
     posix_table["lower"] = list(s)
 
-    # Print: [\x20-\x7E]
+    # `Print: [\x20-\x7E]`
     s = set([x for x in range(0x20, 0x7E + 1)])
     posix_table["print"] = list(s)
 
-    # Punct: [!\"\#$%&'()*+,\-./:;<=>?@\[\\\]^_`{|}~]
+    # `Punct: [!\"\#$%&'()*+,\-./:;<=>?@\[\\\]^_`{|}~]`
     s = set([x for x in range(0x21, 0x2f + 1)])
     s |= set([x for x in range(0x3a, 0x40 + 1)])
     s |= set([x for x in range(0x5b, 0x60 + 1)])
     s |= set([x for x in range(0x7b, 0x7e + 1)])
     posix_table["punct"] = list(s)
 
-    # Space: [ \t\r\n\v\f]
+    # `Space: [ \t\r\n\v\f]`
     s = set([x for x in range(0x09, 0x0d + 1)] + [0x20])
     posix_table["space"] = list(s)
 
-    # Upper: [A-Z]
+    # `Upper: [A-Z]`
     s = set([x for x in range(0x41, 0x5a + 1)])
     posix_table["upper"] = list(s)
 
-    # XDigit: [A-Fa-f0-9]
+    # `XDigit: [A-Fa-f0-9]`
     s = set([x for x in range(0x30, 0x39 + 1)])
     s |= set([x for x in range(0x41, 0x46 + 1)])
     s |= set([x for x in range(0x61, 0x66 + 1)])
     posix_table["xdigit"] = list(s)
 
     # Convert characters values to ranges
-    char2range(posix_table, binary=binary)
+    char2range(posix_table, is_bytes=is_bytes)
 
     with codecs.open(output, 'a' if append else 'w', 'utf-8') as f:
         if not append:
-            f.write(HEADER)
-        # Write out the unicode properties
+            f.write(HEADER.format(UNIVERSION))
+        # Write out the Unicode properties
         f.write('%s_posix_properties = {\n' % prefix)
         count = len(posix_table) - 1
         i = 0
@@ -663,31 +739,31 @@ def gen_posix(output, binary=False, append=False, prefix=""):
 def gen_uposix(table, posix_table):
     """Generate the posix table and write out to file."""
 
-    # Alnum: [\p{L&}\p{Nd}]
+    # `Alnum: [\p{L&}\p{Nd}]`
     s = set(table['l']['c'] + table['n']['d'])
     posix_table["posixalnum"] = list(s)
 
-    # Alpha: [\p{L&}]
+    # `Alpha: [\p{L&}]`
     s = set(table['l']['c'])
     posix_table["posixalpha"] = list(s)
 
-    # ASCII: [\x00-\x7F]
+    # `ASCII: [\x00-\x7F]`
     s = set([x for x in range(0, 0x7F + 1)])
     posix_table["posixascii"] = list(s)
 
-    # Blank: [\p{Zs}\t]
+    # `Blank: [\p{Zs}\t]`
     s = set(table['z']['s'] + [0x09])
     posix_table["posixblank"] = list(s)
 
-    # Cntrl: [\p{Cc}]
+    # `Cntrl: [\p{Cc}]`
     s = set(table['c']['c'])
     posix_table["posixcntrl"] = list(s)
 
-    # Digit: [\p{Nd}]
+    # `Digit: [\p{Nd}]`
     s = set(table['n']['d'])
     posix_table["posixdigit"] = list(s)
 
-    # Graph: [^\p{Z}\p{C}]
+    # `Graph: [^\p{Z}\p{C}]`
     s = set()
     for table_name in ('z', 'c'):
         for sub_table_name in table[table_name]:
@@ -695,11 +771,11 @@ def gen_uposix(table, posix_table):
                 s |= set(table[table_name][sub_table_name])
     posix_table["^posixgraph"] = list(s)
 
-    # Lower: [\p{Ll}]
+    # `Lower: [\p{Ll}]`
     s = set(table['l']['l'])
     posix_table["posixlower"] = list(s)
 
-    # Print: [\P{C}]
+    # `Print: [\P{C}]`
     s = set()
     for table_name in ('c',):
         for sub_table_name in table[table_name]:
@@ -707,7 +783,7 @@ def gen_uposix(table, posix_table):
                 s |= set(table[table_name][sub_table_name])
     posix_table["^posixprint"] = list(s)
 
-    # Punct: [\p{P}\p{S}]
+    # `Punct: [\p{P}\p{S}]`
     s = set()
     for table_name in ('p', 's'):
         for sub_table_name in table[table_name]:
@@ -715,7 +791,7 @@ def gen_uposix(table, posix_table):
                 s |= set(table[table_name][sub_table_name])
     posix_table["posixpunct"] = list(s)
 
-    # Space: [\p{Z}\t\r\n\v\f]
+    # `Space: [\p{Z}\t\r\n\v\f]`
     s = set()
     for table_name in ('z',):
         for sub_table_name in table[table_name]:
@@ -724,11 +800,11 @@ def gen_uposix(table, posix_table):
     s |= set([x for x in range(0x09, 0x0e)])
     posix_table["posixspace"] = list(s)
 
-    # Upper: [\p{Lu}]
+    # `Upper: [\p{Lu}]`
     s = set(table['l']['u'])
     posix_table["posixupper"] = list(s)
 
-    # XDigit: [A-Fa-f0-9]
+    # `XDigit: [A-Fa-f0-9]`
     s = set([x for x in range(0x30, 0x39 + 1)])
     s |= set([x for x in range(0x41, 0x46 + 1)])
     s |= set([x for x in range(0x61, 0x66 + 1)])
@@ -749,15 +825,10 @@ def gen_alias(enum, binary, output, ascii_props=False, append=False, prefix=""):
     divider_re = re.compile(r'#\s*=+\s*$')
     posix_props = ('alnum', 'blank', 'graph', 'print', 'xdigit')
     toplevel = (
-        'catalog', 'enumerated', 'numeric', 'eastasianwidth',
-        'linebreak', 'hangulsyllabletype', 'decompositiontype',
-        'wordbreak', 'sentencebreak', 'graphemeclusterbreak',
-        'joiningtype', 'joininggroup', 'numerictype',
-        'numericvalue', 'canonicalcombiningclass', 'age',
-        'nfcquickcheck', 'nfdquickcheck', 'nfkcquickcheck', 'nfkdquickcheck'
+        'catalog', 'enumerated', 'numeric', 'miscellaneous'
     )
 
-    with open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'PropertyAliases.txt'), 'r') as uf:
+    with codecs.open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'PropertyAliases.txt'), 'r', 'utf-8') as uf:
         div = False
         capture = False
         name = None
@@ -796,7 +867,7 @@ def gen_alias(enum, binary, output, ascii_props=False, append=False, prefix=""):
                     for d in data[1:]:
                         alias[name][d] = data[0]
 
-    with open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'PropertyValueAliases.txt'), 'r') as uf:
+    with codecs.open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'PropertyValueAliases.txt'), 'r', 'utf-8') as uf:
         for line in uf:
             m = alias_re.match(line)
             if m:
@@ -806,7 +877,7 @@ def gen_alias(enum, binary, output, ascii_props=False, append=False, prefix=""):
                 line_re = re.compile(r'%s\s*;' % m.group(2), re.I)
             if gather and line_re.match(line):
                 data = [format_name(x) for x in line.split('#')[0].split(';')]
-                if current_category in ('sc', 'blk', 'dt', 'jg', 'sb', 'wb', 'lb', 'gcb', 'nt'):
+                if current_category in ('sc', 'blk', 'dt', 'jg', 'sb', 'wb', 'lb', 'gcb', 'nt', 'inpc', 'inmc', 'insc'):
                     data[1], data[2] = data[2], data[1]
                 elif current_category == 'age' and UNIVERSION_INFO < (6, 1, 0):
                     if data[2] == 'unassigned':
@@ -834,7 +905,7 @@ def gen_alias(enum, binary, output, ascii_props=False, append=False, prefix=""):
 
     with codecs.open(output, 'a' if append else 'w', 'utf-8') as f:
         if not append:
-            f.write(HEADER)
+            f.write(HEADER.format(UNIVERSION))
         f.write('%s_alias = {\n' % prefix)
         count = len(alias) - 1
         i = 0
@@ -900,10 +971,18 @@ def gen_properties(output, ascii_props=False, append=False):
         'alias': os.path.join(output, 'alias.py')
     }
 
+    files['scx'] = os.path.join(output, 'scriptextensions.py')
+    files['insc'] = os.path.join(output, 'indicsyllabiccategory.py')
+    files['bpt'] = os.path.join(output, 'bidipairedbrackettype.py')
+    files['inpc'] = os.path.join(output, 'indicpositionalcategory.py')
+
+    if UNIVERSION_INFO >= (11, 0, 0):
+        files['vo'] = os.path.join(output, 'verticalorientation.py')
+
     prefix = "ascii" if ascii_props else 'unicode'
 
-    # L& or Lc won't be found in the table,
-    # so intialize 'c' at the start. & will have to be converted to 'c'
+    # `L&` or `Lc` won't be found in the table,
+    # so initialize 'c' at the start. `&` will have to be converted to 'c'
     # before sending it through.
     categories = [
         'generalcategory', 'script', 'block',
@@ -913,6 +992,13 @@ def gen_properties(output, ascii_props=False, append=False):
         'joininggroup', 'numerictype', 'numericvalue',
         'canonicalcombiningclass', 'age'
     ]
+    categories.append('scriptextensions')
+    categories.append('indicsyllabiccategory')
+    categories.append('bidipairedbrackettype')
+    categories.append('indicpositionalcategory')
+
+    if UNIVERSION_INFO >= (11, 0, 0):
+        categories.append('verticalorientation')
     if ascii_props:
         print('=========Ascii Tables=========')
     else:
@@ -922,7 +1008,7 @@ def gen_properties(output, ascii_props=False, append=False):
     all_chars = ALL_ASCII if ascii_props else ALL_CHARS
     table = {'l': {'c': []}}
     itable = {'l': {}}
-    with open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'UnicodeData.txt'), 'r') as uf:
+    with codecs.open(os.path.join(HOME, 'unicodedata', UNIVERSION, 'UnicodeData.txt'), 'r', 'utf-8') as uf:
         for line in uf:
             data = line.strip().split(';')
             if data:
@@ -953,15 +1039,20 @@ def gen_properties(output, ascii_props=False, append=False):
     gen_blocks(files['blk'], ascii_props, append, prefix)
 
     # Generate Unicode scripts
-    print('Building: Scripts')
-    gen_enum(
-        'Scripts.txt', 'scripts', files['sc'], notexplicit='zzzz',
-        ascii_props=ascii_props, append=append, prefix=prefix
+    print('Building: Scripts & Script Extensions')
+    gen_scripts(
+        'Scripts.txt', 'ScriptExtensions.txt', 'scripts', 'script_extensions', files['sc'], files['scx'],
+        notexplicit='zzzz', ascii_props=ascii_props, append=append, prefix=prefix
     )
 
     # Generate Unicode bidi classes
     print('Building: Bidi Classes')
     gen_bidi(files['bc'], ascii_props, append, prefix)
+    print('Building: Bidi Paired Bracket Type')
+    gen_enum(
+        'BidiBrackets.txt', 'bidi_paired_bracket_type', files['bpt'], notexplicit='n',
+        field=2, ascii_props=ascii_props, append=append, prefix=prefix
+    )
 
     # Generate Unicode binary
     print('Building: Binary')
@@ -969,7 +1060,7 @@ def gen_properties(output, ascii_props=False, append=False):
 
     # Generate posix table and write out to file.
     print('Building: Posix')
-    gen_posix(files['posix'], binary=ascii_props, append=append, prefix=prefix)
+    gen_posix(files['posix'], is_bytes=ascii_props, append=append, prefix=prefix)
 
     print('Building: Age')
     gen_age(files['age'], ascii_props, append, prefix)
@@ -1004,6 +1095,18 @@ def gen_properties(output, ascii_props=False, append=False):
         ascii_props=ascii_props, append=append, prefix=prefix
     )
 
+    print('Building: Indic Positional Category')
+    gen_enum(
+        'IndicPositionalCategory.txt', 'indic_positional_category', files['inpc'], notexplicit='na',
+        ascii_props=ascii_props, append=append, prefix=prefix
+    )
+
+    print('Building: Indic Syllabic Category')
+    gen_enum(
+        'IndicSyllabicCategory.txt', 'indic_syllabic_category', files['insc'], notexplicit='other',
+        ascii_props=ascii_props, append=append, prefix=prefix
+    )
+
     print('Building: Hangul Syllable Type')
     gen_enum(
         'HangulSyllableType.txt', 'hangul_syllable_type', files['hst'], notexplicit='na',
@@ -1046,20 +1149,27 @@ def gen_properties(output, ascii_props=False, append=False):
     print('Building: NF* Quick Check')
     categories.extend(gen_nf_quick_check(files['qc'], ascii_props, append, prefix))
 
+    if UNIVERSION_INFO >= (11, 0, 0):
+        print('Building: Vertical Orientation')
+        gen_enum(
+            'VerticalOrientation.txt', 'vertical_orientation', files['vo'], notexplicit='R',
+            ascii_props=ascii_props, append=append, prefix=prefix
+        )
+
     if not ascii_props:
         print('Building: Aliases')
         gen_alias(categories, binary, files['alias'], ascii_props, append, prefix)
 
     # Convert char values to string ranges.
-    char2range(table, binary=ascii_props)
-    char2range(itable, binary=ascii_props, invert=False)
+    char2range(table, is_bytes=ascii_props)
+    char2range(itable, is_bytes=ascii_props, invert=False)
     for k1, v1 in itable.items():
         table[k1]['^'] = v1['^']
 
     with codecs.open(files['gc'], 'a' if append else 'w', 'utf-8') as f:
         if not append:
-            f.write(HEADER)
-        # Write out the unicode properties
+            f.write(HEADER.format(UNIVERSION))
+        # Write out the Unicode properties
         f.write('%s_properties = {\n' % prefix)
         count = len(table) - 1
         i = 0
@@ -1082,7 +1192,7 @@ def gen_properties(output, ascii_props=False, append=False):
 
     if not append:
         with codecs.open(os.path.join(output, '__init__.py'), 'w') as f:
-            f.write(HEADER)
+            f.write(HEADER.format(UNIVERSION))
             for x in sorted(files):
                 f.write('from .%s import *  # noqa\n' % os.path.basename(files[x])[:-3])
 
@@ -1103,19 +1213,34 @@ def build_ascii_property_table(output):
     gen_properties(output, ascii_props=True, append=True)
 
 
-def build_tables(output):
+def build_tables(output, version=None):
     """Build output tables."""
 
+    set_version(version)
     build_unicode_property_table(output)
     build_ascii_property_table(output)
 
 
+def set_version(version):
+    """Set version."""
+
+    global UNIVERSION
+    global UNIVERSION_INFO
+
+    if version is None:
+        version = unicodedata.unidata_version
+
+    UNIVERSION = version
+    UNIVERSION_INFO = tuple([int(x) for x in UNIVERSION.split('.')])
+
+
 if __name__ == '__main__':
     import argparse
 
     parser = argparse.ArgumentParser(prog='unipropgen', description='Generate a unicode property table.')
     parser.add_argument('--version', action='version', version="%(prog)s " + __version__)
+    parser.add_argument('--unicode-version', default=None, help='Force a specific Unicode version.')
     parser.add_argument('output', default=None, help='Output file.')
     args = parser.parse_args()
 
-    build_tables(args.output)
+    build_tables(args.output, args.unicode_version)
